%  PSD_est_AR_cov.m
%
% This program computes the AR spectral estimator using the covariance
% method to estimate the parameters.
%
% Input parameters:
% 
%   x    - N x 1 array of real data
%   p    - AR model order desired
%   plotit - set equal to 1 if PSD plot desired, if not, set = 0
%   Nf     - number of samples of PSD to be plotted
%   plotdB - set equal to 1 for vertical axis in dB
%   yaxismin - minimum value of yaxis in dB (for autoscaling
%              leave blank)
%   yaxismax - maximum value of yaxis in dB (for autoscaling
%              leave blank)
%
%  Output parameters:
%
%   freq   - array of size Nf x 1 containing the frequency samples
%   PARdB - array of size Nf x 1 containing the AR PSD values in dB
%
% Notes: use function [freq,PARdB]=PSD_est_AR(x,p,plotit,Nf,plotdB,yaxismin,yaxismax) 
% to input the optional parameters yaxismin, yaxismax
%
% External subprograms required: AR_par_est_cov.m
%
 function [freq,PARdB]=PSD_est_AR_cov(x,p,plotit,Nf,plotdB,varargin)
   freq=[0:Nf-1]'/Nf-0.5;  % compute frequency values
   [ahat,sig2uhat]=AR_par_est_cov(x,p); % estimate AR parameters using the 
                                  % covariance method
   A=zeros(Nf,1);PAR=zeros(Nf,1);
   for i=1:length(freq)
       A(i,1)=1;
       for k=1:p
           A(i,1)=A(i,1)+ahat(k)*exp(-j*2*pi*freq(i)*k);
       end
       PAR(i,1)=sig2uhat/(abs(A(i))^2);
   end
   PARdB=10*log10(PAR);  % convert to dB quantities
   if plotit==1
       if plotdB==1
   plot(freq,PARdB)  % plot the AR PSD estimate
   xlabel('frequency')
   ylabel('AR PSD estimate (dB)')
   grid
       else
     plot(freq,PAR)  % plot the AR PSD estimate
   xlabel('frequency')
   ylabel('AR PSD estimate (dB)')
   grid  
       end
   if nargin==6
      axis([-0.5 0.5 varargin{1} varargin{2}])
   end
   end
      
   


