% PSD_est_mvse.m
%
% This program computes the minimum variance spectral estimator as given by
% (11.6).
% 
% Input parameters:
%
%   x    - N x 1 array of real data
%   p    - size of the p x p autocorrelation matrix to use
%   Nf   - number of frequency points in [-1/2,1/2] interval to compute
%          the spectral estimate
%   plotit - set equal to 1 to plot the PSD estimate
%   plotdB - set equal to 1 if vertical axis to be plotted in dB
%   yaxismin - minimum value of yaxis in dB (for autoscaling
%              leave blank)
%   yaxismax - maximum value of yaxis in dB (for autoscaling
%              leave blank)
%
%  Output parameters:
%
%   freq   - array of size Nf x 1 containing the frequency samples
%   Pmvse - array of size Nf x 1 containing the PSD values.
%
% Notes: use function [freq,Pmvse]=PSD_est_mvse(x,p,plotit,plotdB,yaxismin,yaxismax) 
% to input the optional parameters yaxismin, yaxismax
%
  function [freq,Pmvse]=PSD_est_mvse(x,p,Nf,plotit,plotdB,varargin)
  N=length(x);
for i=1:p  % compute autocorrelation matrix estimate (uses "forward-backward"
           % estimate)
    for j=1:p
        R(i,j)=0;
        for n=p:N-1
            R(i,j)=R(i,j)+(1/(N-p))*(x(n-i+1)*x(n-j+1));  
        end
        for n=0:N-1-p
            R(i,j)=R(i,j)+(1/(N-p))*(x(n+i+1)*x(n+j+1));
        end
    end
end
Rinv=inv(R); % invert autocorrelation matrix estimate
freq=[0:Nf-1]'/Nf-0.5;
for i=1:length(freq)
    e=exp(sqrt(-1)*2*pi*freq(i)*[0:p-1]');
    Pmvse(i,1)=real(p./(e'*Rinv*e));  % compute MVSE
end
if plotit==1
       if plotdB==1
       PmvsedB=10*log10(Pmvse);  % convert to dB quantities
       plot(freq,PmvsedB)  % plot the spectral estimate
       xlabel('frequency')
       ylabel('minimum variance spectral estimate (dB)')
       grid
       else
       plot(freq,Pmvse)  % plot the spectral estimate
       xlabel('frequency')
       ylabel('minimum variance spectral estimate')
       grid
       end
   if nargin==6
      axis([-0.5 0.5 varargin{1} varargin{2}])
   end
   end
        
