% FSSP3exer13_3.m
%
%  This program implements the MLE (see (13.6)) and FFT frequency 
%  estimators described in Chapter 13.  Additionally the CRLB is used 
%  as a benchmark.
%
%  External function subprograms required:
%   ARgendata_complex.m
%
clear all
close all
randn('state',0) % initialize Gaussian random number generator
Q=15;  
f0=[0.1:0.05:0.45]';  % set up values of center frequency to minimize
                      % objective function over
r=1./(1+(pi*f0)/Q);   % relate the pole radius to the center frequency -
                      % see (13.3)
a1=-r.*exp(j*2*pi*f0); % set complex AR filter parameter
sig2u=1-abs(a1).^2;   % set excitation white noise variance so that
                      % the average power in the AR process is 1
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
N=300;                 %  THIS IS INCREASED DATA RECORD LENGTH
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
ntrials=10000;        % number of trials to use in Monte Carlo simulation
                      %  should increase number until the estimator
                      %  variance curve appears smooth
f0try=[0.05:0.001:0.5]'; % set up range for center frequency to be computed
% for objective function.  Note that it is slightly larger than range of
% known values of f0.  This is because otherwise, if the true value is say
% f0=0.1 and we compute the objective function starting at f0try=0.1, the
% error cannot be less than zero.  This causes a bias in the estimator and
% nullifies the use of the CRLB.  In practice, we seldom know exactly the
% range of f0 since it is usually a guess anyway. Furthermore, it makes
% the comparison with the periodogram fairer, which will search over
% [0,0.5] frequency range
rtry=1./(1+(pi*f0try)/Q); % range of possible pole radius
Nfft=2048;  % FFT size to be used for periodogram
freq=[0:Nfft-1]'/Nfft;  % frequencies to evaluate periodogram.  We will 
                        % only use the ones over the interval [0,0.5]

for k=1:length(f0) % start loop over center frequencies  
for i=1:ntrials  % start loop over Monte Carlo trials
x=ARgendata_complex(a1(k),sig2u(k),N); % generate complex AR signal
per=(1/N)*abs(fft(x,Nfft)).^2; % compute periodogram
[mm II]=max(per(1:Nfft/2-1));  % find maximum and location of maximum over
                               % the frequency range [0,0.5]
f0hat_per(i,k)=freq(II);  % find the frequency of maximum
rhohat=sum(conj(x(1:N-1)).*x(2:N))/(x'*x); % compute normalized autocorrelation
                                        % at lag k=1 - see (13.8)
for l=1:length(f0try)  % start loop for objective function
J(l,1)=1+rtry(l)^2-2*rtry(l)*real(rhohat*exp(-j*2*pi*f0try(l))); % see (13.6)
end
[mm I]=min(J);  % find minimum and location of minimum of objective function
f0hat(i,k)=f0try(I); % find MLE of center frequency
end
end

for k=1:length(f0)  % compute the variances of MLE and periodogram
  var(k,1)=(1/ntrials)*sum( (f0hat(:,k)-mean(f0hat(:,k))).^2  ) ;
  var_per(k,1)=(1/ntrials)*sum( (f0hat_per(:,k)-mean(f0hat_per(:,k))).^2  ) ;
end

f0lb=[0.1:0.01:0.45]'; % set up frequency range over which to compute CRLB
rlb=1./(1+(pi*abs(f0lb))/Q);
den=2*N*((pi^2*rlb.^2)/Q^2+4*pi^2*rlb.^2);
crlb=(1-rlb.^2)./den; % compute CRLB - see (13.4)

figure  % the rest of this code is for plotting 
axes('position',[0.2,0.2,4*(0.375/3),4*(0.375/3)])
   line(f0,var,'linewidth',1.5)
   line(f0,var_per,'linewidth',1.5,'linestyle','--')
   line(f0lb,crlb,'linewidth',1)
  grid
xlabel('$f_0$','interpreter','Latex')
ylabel('Variance')
text(0.46,max(crlb),'CRLB')
text(0.46,max(var)+0.03e-4,'MLE')
text(0.46,max(var_per),'FFT')
axis([0.1 0.5 0 2e-4])
set(gca,'YTick',[0:0.25:2]'*1e-4)
title('FSSP3fig13\_2\_exer')



